from wxPython.wx import *

from Thuban.UI.graph import *

class Sliders(Graph):
    # Everthing internal to Sliders is handled in screen coordinates.
    # Input is normalized to [0,1] as with everything derived from Graph
    # Output, idem.
    def __init__(self, parent, pos=None, width=400, height=10):
        Graph.__init__(self, parent,width,height)
        self.marker_bitmap = wxBitmap("Resources/Bitmaps/triangle_marker.xpm",wxBITMAP_TYPE_XPM)#XXX get proper path!!!
        self.marker_dc = wxMemoryDC()
        self.marker_dc.SelectObject(self.marker_bitmap)
        self.status='idle'

        self.leftoffset = self.border * self.w
        self.scalefactor = self.w/(1.0+2.0*self.border)

        if len(pos)>0:
            self.pos = [ p*self.scalefactor+self.leftoffset for p in pos ]
        else:
            self.pos = [ pos*self.scalefactor+self.leftoffset ]

        EVT_LEFT_DOWN( self, self.OnLeftDown )
        EVT_LEFT_UP( self, self.OnLeftUp )
        EVT_MOTION( self, self.OnMotion )
        EVT_LEAVE_WINDOW( self, self.OnLeaveWindow )
        EVT_LEFT_DCLICK( self, self.OnLeftDClick)

    def GetList(self):
        l = [ (p-self.leftoffset)/self.scalefactor for p in self.pos ]
        l.sort()
        return l

    def findSlider(self,xy):
        i=0
        found=False
        for p in self.pos:
            if abs(xy.x-p)<xy.y/2.0+1: #a "triangular" search
                found = True
                break
            i+=1
        return (found,i)

    def getNum(self):
        return len(self.pos)

    def OnLeftDClick(self,event):
        xy = event.GetPosition()
        (found,i) = self.findSlider( xy )
        if found:
            try:
                self.pos = self.pos[:i]+self.pos[i+1:]
                self.refresh()
            except:
                pass

    def OnLeftDown(self,event):
        xy = event.GetPosition()
        (found,i) = self.findSlider( xy )
        if found:
            self.status = "selected"
            self.x = xy.x
            self.index = i
        else:
            self.pos.append( xy.x )
            self.refresh()
            self.x = xy.x
            self.index = len(self.pos)
            self.status = "selected"
        
    def OnLeftUp(self,event):
        evx = event.GetPosition().x
        if self.status in ("selected",):
            if abs(evx-self.x)<2:
                pass
            else:
                self.pos[self.index] = evx
                self.refresh()
        self.status = "idle"

    def OnMotion(self,event):
        pass

    def OnLeaveWindow(self,event):
        self.status = "idle" 

    def draw_marker(self,x,drawfunc=wxCOPY):
        dc = wxPaintDC(self)
        dc.Blit( int(x-4),0,
                 self.marker_bitmap.GetWidth(),self.marker_bitmap.GetHeight(),
                 self.marker_dc,0,0,drawfunc)

    def inspect(self):
        print "pos = ",self.pos
        print "status = ",self.status
        if self.status == "selected":
            print "   self.x=",self.x
            print "   self.index=",self.index

    def refresh(self, dc=None):
        if dc is None:
            dc=wxPaintDC(self)
            dc.Clear()
        for p in self.pos:
            self.draw_marker( p )


